//+------------------------------------------------------------------+
//|                                           PuriaM2_*.mq4 Modified |
//+------------------------------------------------------------------+
#property copyright " Oleg Tsiliuric"
#property link      "olej@front.ru"
#include <stdlib.mqh>                         //  
#include <strlib.mqh>
#include <mmlib.mqh>                         
#define Vers "7, 26.02.2011" 

extern double FixLotSize = 0.1;               //   ,  0 -  RiskPercent
extern int RiskPercent   = 3;                 // MAX.   -  1 : 0-100% (  FixLotSize = 0.0)
extern int StopLossPts   = 60;                //   Digits=4!
extern int TakeProfitPts = 200;               //   Digits=4!
#define SlippagePts 3                         //   / ()   
extern int MovingPeriod1 = 16;
extern int MovingPeriod2 = 70;
extern int MovingPeriod3 = 200;
extern int DebugLevel    = 2;
extern string ConfigureFile = "PuriaM2.cfg";  //   "" -   
//    ():
#define MM_nothing         0                  //  
#define MM_proportion      1                  //   
#define MM_rajan_jones     2                  //    
//#define MM_from_MA         3                  //       
//#define MM_martingale      4                  //      
extern int MM_method         = MM_nothing;    //   
extern double MM_parameter_1 = 0.0,
              MM_parameter_2 = 0.0;
extern bool Bezubitok    = false;

#define Magic 33445566
static bool Working = true;                   //   
int ErrOpen = 0, ErrClose = 0, ErrModify = 0; //      
//===================================================================
void deinit() {
   if( DebugLevel > 1 ) {
      Print( "   ?" );
      Print( " : =", ErrOpen, 
             ", =", ErrClose, ", =", ErrModify );
   }
}
//===================================================================
double LotBeginSize,                         //     
       AccountStart;                         //    
int StopLoss, TakeProfit,                    //   SL/TP   Digits
    Slippage;                                //    ,   Digits
void init() {
   static bool bFirst = true;
   if( DebugLevel > 0 ) {
      if( bFirst ) Print( " :  ", Vers );
      else Print( " !" );      
   }
   if( bFirst ) bFirst = false;              //  , 1- 
   Working = TestParams();                   //   
   if( !Working ) return;
   ErrOpen = 0; ErrClose = 0; ErrModify = 0;
   StopLoss = PointToDigits( StopLossPts );  //  -  Digits
   TakeProfit = PointToDigits( TakeProfitPts );
   Slippage =  PointToDigits( SlippagePts );                        
   string Symb = Symbol();
   AccountStart = AccountFreeMargin();       //     
   double Zalog = MarketInfo( Symb, MODE_MARGINREQUIRED ); //   1- . 
   if( !CompareDoubles( FixLotSize, 0.0 ) )  //    
      LotBeginSize = FixLotSize;
   else                                      //       
      LotBeginSize = ( AccountStart * RiskPercent / 100. ) / Zalog;
   if( DebugLevel > 0 ) {
      #define VALLEN 3
      string OrdCurrency = StringSubstr( Symb, 0, VALLEN ),
             CotCurrency = StringSubstr( Symb, VALLEN, VALLEN ),
             DepCurrency = AccountCurrency();
      Print( ".  = ", DoubleToStr( MarketInfo( Symb, MODE_LOTSIZE ), 0 ), " ", OrdCurrency,
             ",  .  = ", Zalog, " ", DepCurrency,
             ",  = ", AccountStart, " ", DepCurrency );
      Print( "  = ", LotBeginSize,
             ",  slippage : ", Slippage,
             ",  : sl : ", StopLoss, " , tp : ", TakeProfit ); 
   }
   return; 
}
//-------------    --------------------
bool TestParams() {
   string ErrTail = " :  !";
   if( CompareDoubles( FixLotSize, 0.0 ) == true &&  //    
       ( RiskPercent <= 0 || RiskPercent > 100 ) ) {
      Alert( "RiskPercent     0...100", ErrTail );
      return ( false );
   }
   switch( MM_method ) {                                
      case MM_rajan_jones:
         if( MM_parameter_1 < 0.0 || MM_parameter_1 > 1.0 ) {
            Alert( "MM_parameter_1        0...1", ErrTail );
            return ( false );
         }
         break;    
/*      case MM_from_MA:
      default:
         return ( true );      */
   }
   return ( true );
}
//===================================================================
void start() {
   static int nBars = 0;
   static bool Begin = true;
   static bool BuyOne = true, SellOne = true;
   if( !Working || Bars<100 || IsTradeAllowed()==false ) return;
   if( Begin==false ) {                                //     ! 
      if( nBars == Bars )                              //      ...
         return;
      else {
         bool First = ( nBars == 0 );
         nBars = Bars;
         if( First ) return;
         if( DebugLevel > 2 ) Print( "  : ", nBars );      
      }
   }
   //<===        !
   double ma1, ma2, ma3, macd;
   ma1  = iMA( NULL, 0, MovingPeriod1, 0, MODE_EMA, PRICE_CLOSE, 1 );
   ma2  = iMA( NULL, 0, MovingPeriod2, 0, MODE_LWMA, PRICE_LOW, 1 );
   ma3  = iMA( NULL, 0, MovingPeriod3, 0, MODE_LWMA, PRICE_LOW, 1 );
   macd = iMACD( NULL, 0, 15, 26, 1, PRICE_CLOSE, MODE_MAIN, 1 );
   if( Begin ) {                                          //    MA
      if( ma1 > ma2 && ma1 > ma3 ) {
         BuyOne = false; SellOne = true; Begin = false;
      }
      if( ma1 < ma2 && ma1 < ma3 ) {
         SellOne = false; BuyOne = true; Begin = false;
      }  
      return;
   }
   if( DebugLevel > 0 ) {                                 //   
      int Draft = iCustom( NULL, 0, "iPuria", 
                           MovingPeriod1, MovingPeriod2, MovingPeriod3, 
                           0, 0 );
   }                           
//------------  (SL)    ---------------
   int Dist = MarketInfo( Symbol(), MODE_STOPLEVEL );     //   -
   if( StopLoss < Dist ) {
      StopLoss = Dist;                                    //  
      if( DebugLevel > 0 ) Print( " : sl : ", Dist );
   }
   if( TakeProfit < Dist ) {
      TakeProfit = Dist;                                  //  
      if( DebugLevel > 0 ) Print( " : tp : ", Dist );
   }
   PrevOrdersModify();                                    //   -   
//----------------     -----------------------
#define TRY_COUNTER 3
   int Try = TRY_COUNTER;                                 //    
   double Lot;
   while( Try > 0 ) {
      int Ticket;  
      int ErrorChr;                                       //    
      Try--;
      RefreshRates();                                     //        
      if( ma1 < ma2 && ma1 < ma3 ) {                      // ... SELL 
         BuyOne = true;                                   //   
         if( macd < 0 && SellOne ) {
            OppositClose( OP_SELL );                      //    
            Lot = Lots( OP_SELL );
            if( DebugLevel > 1 )                          //  
               Print( ": cmd=", OP_SELL, " -> ", Lot,
                " dep: ", DoubleToStr( AccountFreeMargin(), 0 ),
                ", at: ", DoubleToStr( Bid, Digits ),
                ", tp: ", DoubleToStr( Ask - TakeProfit * Point, Digits ),
                ", sl: ", DoubleToStr( Ask + StopLoss * Point, Digits )
                    );
         RefreshRates();
			Ticket = OrderSend( Symbol(), OP_SELL, Lot, 
                          NormalizeDouble( Bid, Digits ), Slippage,
                          0,
                          0,
                          "", Magic, 0, Red );
            if( Ticket > 0 ) {
               OrderSelect(Ticket, SELECT_BY_TICKET);
			      OrderModify(OrderTicket(), OrderOpenPrice(), OrderOpenPrice() + StopLoss * Point, 
			                  OrderOpenPrice() - TakeProfit * Point, 0, CLR_NONE);
               SellOne = false; 
               return;                                    // ...
            }
            else {    
               ErrOpen++;            
               ErrorChr = Fun_Error( GetLastError() );
               if( ErrorChr==1 ) continue;                //   -  
               else if( ErrorChr==-1 ) Working = false;   //  
            }
         }
      } // ... SELL
      if( ma1 > ma2 && ma1 > ma3 ) {                       // ... BUY 
         SellOne = true;                                   //         
         if( macd > 0 && BuyOne ) { 
            OppositClose( OP_BUY );                        //    
            Lot = Lots( OP_BUY );
            if( DebugLevel > 1 )                           //  
               Print( ": cmd=", OP_BUY, " -> ", Lot,
                " dep: ", DoubleToStr( AccountFreeMargin(), 0 ),
                ", at: ", DoubleToStr( Ask, Digits ),
                ", tp: ", DoubleToStr( Bid + TakeProfit * Point, Digits ),
                ", sl: ", DoubleToStr( Bid - StopLoss * Point, Digits )
                    );
         RefreshRates();
			Ticket = OrderSend( Symbol(), OP_BUY, Lot, 
                          NormalizeDouble( Ask, Digits ), Slippage,
                          0, 
                          0,
                          "", Magic, 0, Blue );                                
            if(  Ticket > 0 ) {
               OrderSelect(Ticket, SELECT_BY_TICKET);
			      OrderModify(OrderTicket(), OrderOpenPrice(), OrderOpenPrice() - StopLoss * Point, 
			                  OrderOpenPrice() + TakeProfit * Point, 0, CLR_NONE); 
               BuyOne = false;
               return;                                     // ...
            }   
            else {    
               ErrOpen++;
               ErrorChr = Fun_Error( GetLastError() );
               if( ErrorChr==1 ) continue;                 //   -  
               else if( ErrorChr==-1 ) Working = false;    //  
            }
         }
      } // ... BUY
      return;                                              //   start()   
   }
}
//===================================================================
//---- :      !  ------
double Lots( int Ops ) {
   double Ratio = AccountFreeMargin() / AccountStart,
          Lots;                               //     
   switch( MM_method ) {
      case MM_nothing: 
         Lots = LotBeginSize;                 //   !
         break;
      case MM_proportion:
         Lots = LotBeginSize * Ratio;
         break;
      case MM_rajan_jones:
         Lots = LotBeginSize * Rajan_Jones( Ratio, MM_parameter_1,
                                  MarketInfo( Symbol(), MODE_MAXLOT ) / LotBeginSize );
         break;       
/*      case MM_from_MA:
         Lots = LotBeginSize;
         if( AboveMaProfit( Ops, Magic ) )    //    
            Lots *= Ratio;
         break;      */
      default:
         Lots = LotBeginSize;                 //   !
         break;              
   }
   return( NormalizeOrderSize( Lots ) ); 
}
//--------------------------------------------------------------------
double Rajan_Jones( double Ratio, double Delta, double HighLim ) {
   if( Ratio < 1.0 ) return( 1.0 );                 //   !
   double Curr = ( Ratio - 1.0 ) / Delta;           //  ""  
   static double Mult[] = { 
      0,   1,   3,   6,  10,                        // Mult[i] = Mult[i-1]+i
     15,  21,  28,  36,  45,
     55,  66,  78,  91, 105,
    120, 136, 153, 171, 190   
   };
   int i, n = ArraySize( Mult );
   double Limit = 1.0;
   for( i = 1; i <  HighLim + 1; i++ ) {
      if( Curr < Limit ) break;
      Limit += ( i + 1 );       // 1, 3, 6, 10... X[i]=X[i-1]+i
//      if( Curr < Mult[ i ] ) break;
   }
   return ( 1.0 + ( i - 1 ) );
}
//--------------------------------------------------------------------
//---  (SL)      -----------
void PrevOrdersModify() {
   if( !Bezubitok ) return;
   for( int j = 0; j < OrdersTotal(); j++ ) {
      if( OrderSelect( j, SELECT_BY_POS )==true ) {          //   
         if( OrderSymbol()!=Symbol() ) continue;             //    
         if( OrderMagicNumber() != Magic ) continue;         //    
         int Tip = OrderType();                              //   
         if( Tip != OP_BUY && Tip != OP_SELL ) continue;     //   
         int Ticket = OrderTicket();                         //   
         double NewLoss;
         if( Tip == OP_SELL ) { // SELL
            if( OrderStopLoss() <= OrderOpenPrice() )        //   
               continue;
            NewLoss = Ask + ( Point * StopLoss );
            if( NewLoss < OrderOpenPrice() ) NewLoss = OrderOpenPrice();
            if( NewLoss < OrderStopLoss() ) ModifyStopLoss( OP_BUY, NewLoss );
         }            
         if( Tip == OP_BUY ) {  // BUY
            if( OrderStopLoss() >= OrderOpenPrice() )        //   
               continue;
            NewLoss = Bid - ( Point * StopLoss );
            if( NewLoss > OrderOpenPrice() ) NewLoss = OrderOpenPrice();
            if( NewLoss > OrderStopLoss() ) ModifyStopLoss( OP_BUY, NewLoss );
         }
      }
   }
}
//--------------------------------------------------------------------
void ModifyStopLoss( int op, double Stop ) {
   double StopLevel = NormalizeDouble( Stop, Digits );    //  -
   int Try = TRY_COUNTER;                                 //   
   while( Try > 0 ) {
      Try--;
      while( !IsTradeAllowed() ) Sleep( 5000 );
      RefreshRates();
      if( OrderModify( OrderTicket(), OrderOpenPrice(),
                       StopLevel , OrderTakeProfit(), 0, Green ) ) {
         return;
      }
      else {
         ErrModify++;      
         int ErrorChr = Fun_Error( GetLastError() );      //    
         if( ErrorChr==1 ) continue;                      //   -  
         else return;                                     //  
      }
   }
}
//----------------    -----------------------
void OppositClose( int Opr ) {
   int j = 0; 
   while( j < OrdersTotal() ) {                              //  for -  !
      j++;   
      if( OrderSelect( j - 1, SELECT_BY_POS ) == true ) {    //  ?
         if( OrderSymbol()!= Symbol() ) continue;            //   
         int Tip = OrderType();                              //   
         if( Tip != OP_BUY && Tip != OP_SELL ) continue;     //   
         if( Tip == Opr ) continue;                          //   
         if( OrderMagicNumber() != Magic ) continue;         //     
         int Ticket = OrderTicket();                         //   
         int Try = TRY_COUNTER;                              //   
         while( Try > 0 ) {
            RefreshRates();                                  //  
            Try--;
//            double Lot = OrderLots();                        //       
            bool Ansv;
            if( DebugLevel > 1 )                             //  
               Print( ": #", Ticket );
            if( Tip == OP_BUY ) 
               Ansv = OrderClose( Ticket, OrderLots(), NormalizeDouble( Bid, Digits ), Slippage );
            if( Tip == OP_SELL ) 
               Ansv = OrderClose( Ticket, OrderLots(), NormalizeDouble( Ask, Digits ), Slippage );
            if( Ansv == true ) {                             //  !
               j = 0;                                        //    !
               break;
            }
            ErrClose++;
            int ErrorChr = Fun_Error( GetLastError() );      //  !
            continue;
         }
      }      
   }
}
//---------------------------------------------------------------------
int Fun_Error( int Err ) {
   Print( ": ", ErrorRus( Err ) );
   return ( ErrorClass( Err ) );
}
//---------------------------------------------------------------------